﻿Public Class Smp
    '****************************************************************************************************************
    '
    '       VB.NET Sample Program                                         5.4802.22.2501
    '
    '****************************************************************************************************************

    '****************************************************************************************************************
    '
    '   Definition of the variable
    '
    '****************************************************************************************************************
    Dim hDev As Integer                                         ' Device Handle
    Dim hUnit As Integer                                        ' Unit Handle
    Dim hInPort As Integer                                      ' IN Port Handle
    Dim hOutPort As Integer                                     ' OUT Port Handle

    '****************************************************************************************************************
    '
    '   Setting the condition of the button in drive condition.
    '
    '****************************************************************************************************************
    Private Sub ButtonDisable()
        Button_Close.Enabled = False
        Button_End.Enabled = False
        Button_Reset.Enabled = False
        Button_DataSet.Enabled = False
        Button_Ccw_Scan.Enabled = False
        Button_Cw_Scan.Enabled = False
        Button_Org.Enabled = False
        Button_Rtn.Enabled = False
        Button_Ccw_Index.Enabled = False
        Button_Cw_Index.Enabled = False
        Button_Demo.Enabled = False
        Button_ClrError.Enabled = False
    End Sub

    '****************************************************************************************************************
    '
    '   Setting the condition of the button in not drive condition.
    '
    '****************************************************************************************************************
    Private Sub ButtonEnable()
        Button_Close.Enabled = True
        Button_End.Enabled = False
        Button_Reset.Enabled = True
        Button_DataSet.Enabled = True
        Button_Ccw_Scan.Enabled = True
        Button_Cw_Scan.Enabled = True
        Button_Org.Enabled = True
        Button_Rtn.Enabled = True
        Button_Ccw_Index.Enabled = True
        Button_Cw_Index.Enabled = True
        Button_Demo.Enabled = True
        Button_ClrError.Enabled = True
    End Sub

    '****************************************************************************************************************
    '
    '   The stoppage factor of DRIVE that is shown with StopCode is displayed.
    '   Parameters StopCode ･･･ The variable where the drive stoppage factor is stored.
    '                           0:PASS  1:FSEND  2:SSEND  3:LSEND  4:SENSOR ERROR  5:ERROR PULSE ERROR  6:ADDRESS ERROR
    '                           7:FUNCTION ERROR(NO MESSAGE)
    '****************************************************************************************************************
    Private Sub StopCodeDisplay(ByVal StopCode As Byte)
        Select Case StopCode
            Case 1
                Label_Message.Text = "FSSTOP was input."

            Case 2
                Label_Message.Text = "SLSTOP was input."

            Case 3
                Label_Message.Text = "LIMIT was input."

            Case 4
                Label_Message.Text = "SENSOR ERROR was detected." & vbCrLf & "Please Click 'Clear Error'."

            Case 5
                Label_Message.Text = "ERROR PULSE ERROR was detected." & vbCrLf & "Please Click 'Clear Error'."

            Case 6
                Label_Message.Text = "ADDRESS ERROR was detected." & vbCrLf & "Please Click 'Clear Error'."
            Case 7

            Case Else
                Label_Message.Text = "Drive ended."
        End Select
    End Sub

    '********************************************************************************************************************
    '
    '   Check Status1 PORT.
    '   Parameter hDevice         ･･･ Device Handle
    '   Return Value              ･･･ TRUE: STATUS1 PORT BUSY BIT = 0 and ERROR BIT = 0
    '                                 FALSE: STATUS1 PORT BUSY BIT = 1 or ERROR BIT = 1
    '********************************************************************************************************************
    Private Function CheckStatus1(ByVal hDevice As Integer) As Boolean
        Dim sResult As New MC07_S_RESULT()
        Dim Status1 As Short
        Dim Data1 As Short
        Dim Cmd As Short

        sResult.Initialize()

        '*** CHECK STATUS1 BUSY BIT = 0 (WAIT TIME = 1000ms)***
        If (MC07_BWaitDriveCommand(hDevice, 1000, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BWaitDriveCommand() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Return False
        End If

        '*** READ STATUS1 ***
        If (MC07_BRStatus1(hDevice, Status1, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BRStatus1() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Return False
        End If

        '*** CHECK STATUS1 ERROR BIT = 0 ***
        If Status1 And &H10S Then
            '*** READ ERROR STATUS ***
            Cmd = &HD1S
            If (MC07_BWDriveCommand(hDevice, Cmd, sResult) = False) Then
                Label_Message.Text = "It failed in MC07_BWDriveCommand(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
                Return False
            End If

            If (MC07_BRDriveData1(hDevice, Data1, sResult) = False) Then
                Label_Message.Text = "It failed in MC07_BRDriveData1() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
                Return False
            End If

            Label_Message.Text = "STATUS1 ERROR = 1 : ERROR STATUS = H'" & Hex(Data1) & vbCrLf _
                                    & "Please click 'Clear Error'"

            Return False

        End If

        Return True
    End Function

    '****************************************************************************************************************
    '
    '   Waits until MCC07 becomes ready condition. Meantime position address of motor is displayed.
    '   Parameter hDevice         ･･･ Device Handle
    '             StopCode        ･･･ Pointer of the variable where the drive stoppage factor is stored.
    '                                 0:PASS  1:FSEND  2:SSEND  3:LSEND  4:SENSOR ERROR  5:ERROR PULSE ERROR  6:ADDRESS ERROR
    '                                 7:FUNCTION ERROR
    '             OriginDriveFlag ･･･ The Flag is shown that ORIGIN DRIVE executing or other DRIVE executing.
    '                                 True:ORIGIN DRIVE executing  False:other DRIVE executing
    '
    '****************************************************************************************************************
    Private Sub ReadyWait(ByVal hDevice As Integer, ByRef StopCode As Byte, ByVal OriginDriveFlag As Boolean)
        Dim sResult As New MC07_S_RESULT()
        Dim Status1 As Short

        sResult.Initialize()
        Do
            System.Windows.Forms.Application.DoEvents()
            Call NowAddressDisplay(hDevice)
            If (MC07_BRStatus1(hDevice, Status1, sResult) = False) Then
                Label_Message.Text = "It failed in MC07_BRStatus1() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
                StopCode = 7
                Exit Sub
            End If
        Loop While (Status1 And &H1S) <> 0

        Call NowAddressDisplay(hDevice)

        If OriginDriveFlag = True Then
            If (MC07_ReadOrgStatus(hDevice, Status1, sResult) = False) Then
                Label_Message.Text = "It failed in MC07_ReadOrgStatus() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
                StopCode = 7
                Exit Sub
            End If
            If Status1 And &H8000S Then
                StopCode = 6
            ElseIf Status1 And &H4000S Then
                StopCode = 5
            ElseIf Status1 And &H2000S Then
                StopCode = 4
            ElseIf Status1 And &H20S Then
                StopCode = 3
            ElseIf Status1 And &H40S Then
                StopCode = 2
            ElseIf Status1 And &H80S Then
                StopCode = 1
            Else
                StopCode = 0
            End If
        Else
            If Status1 And &H20S Then
                StopCode = 3
            ElseIf Status1 And &H40S Then
                StopCode = 2
            ElseIf Status1 And &H80S Then
                StopCode = 1
            Else
                StopCode = 0
            End If
        End If
    End Sub

    '****************************************************************************************************************
    '
    '   Position address of motor is displayed.
    '   Parameter hDevice ･･･ Device Handle
    '
    '****************************************************************************************************************
    Private Sub NowAddressDisplay(ByVal hDevice As Integer)
        Dim sResult As New MC07_S_RESULT()
        Dim WData As Integer
        Dim RData As Integer
        Dim Cmd As Short

        sResult.Initialize()

        '*** ADDRESS COUNTER READ COMMAND ***
        Cmd = &HD8S
        WData = 0
        If (MC07_LWRDrive(hDevice, Cmd, WData, RData, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_LWRDrive(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If
        Label_NowAddress.Text = CStr(RData)
    End Sub
    '****************************************************************************************************************
    '
    '   The processing that the Environment Button was clicked.
    '
    '****************************************************************************************************************
    Private Sub Button_Env_Click(sender As Object, e As EventArgs) Handles Button_Env.Click
        Dim sResult As New MC07_S_RESULT()
        Dim unitInfo As New MC07_S_UNIT_INFO

        sResult.Initialize()
        Label_Message.Text = ""

        If MC07_Environment(MC07_USB_UNIT_0, sResult) = False Then
            Label_Message.Text = "It failed in MC07_Environment() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        If MC07_ReadUnitInfo(MC07_USB_UNIT_0, unitInfo, sResult) = False Then
            Label_Message.Text = "It failed in MC07_ReadUnitInfo() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        If unitInfo.UnitType(0) = &HFFS Then
            Label_Message.Text = "NO CONTROLLER UNIT:Please Check USB connection"
            Exit Sub
        End If

        Button_Env.Enabled = False
        Button_Open.Enabled = True
        Button_Close.Enabled = False
        Button_End.Enabled = True
        Button_Reset.Enabled = False
        Button_FastStop.Enabled = False
        Button_SlowStop.Enabled = False
        Button_DataSet.Enabled = False
        Button_Ccw_Scan.Enabled = False
        Button_Cw_Scan.Enabled = False
        Button_Org.Enabled = False
        Button_Rtn.Enabled = False
        Button_Ccw_Index.Enabled = False
        Button_Cw_Index.Enabled = False
        Button_Demo.Enabled = False
        Button_ClrError.Enabled = False
        Label_NowAddress.Text = "Please open the device."

        Label_Message_ExtIO.Text = "Please Open Extend IO."
        Button_OpenExtIO.Enabled = True
        Button_CloseExtIO.Enabled = False
        Button_ExtLatchClr.Enabled = False
        Button_ReadExtIn0.Enabled = False
        Button_ExtOut0.Enabled = False
        Button_ExtOut1.Enabled = False
        Button_ExtOut2.Enabled = False
        Button_ExtOut3.Enabled = False
    End Sub

    '****************************************************************************************************************
    '
    '   The processing that the Open Button was clicked.
    '
    '****************************************************************************************************************
    Private Sub Command_Open_Click(sender As Object, e As EventArgs) Handles Button_Open.Click
        Dim sResult As New MC07_S_RESULT()

        sResult.Initialize()
        Label_Message.Text = ""
        If (MC07_BOpen(MC07_USB_UNIT_0, MC07_X, hDev, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BOpen() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Button_Env.Enabled = True
            Exit Sub
        End If
        Button_Open.Enabled = False
        Button_Close.Enabled = True
        Button_End.Enabled = False
        Button_Reset.Enabled = True
        Button_FastStop.Enabled = True
        Button_SlowStop.Enabled = True
        Button_DataSet.Enabled = True
        Button_Ccw_Scan.Enabled = True
        Button_Cw_Scan.Enabled = True
        Button_Org.Enabled = True
        Button_Rtn.Enabled = True
        Button_Ccw_Index.Enabled = True
        Button_Cw_Index.Enabled = True
        Button_Demo.Enabled = True
        Button_ClrError.Enabled = True
        NowAddressDisplay(hDev)                                 ' Position address is displayed.
    End Sub

    '****************************************************************************************************************
    '
    '   The processing that the Close Button was clicked.
    '
    '****************************************************************************************************************
    Private Sub Command_Close_Click(sender As Object, e As EventArgs) Handles Button_Close.Click
        Dim sResult As New MC07_S_RESULT()

        sResult.Initialize()
        Label_Message.Text = ""
        '*** DEVICE CLOSE ***
        If (MC07_BClose(hDev, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BClose() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Button_Env.Enabled = True
            Exit Sub
        End If
        Button_Env.Enabled = True
        Button_Open.Enabled = True
        Button_Close.Enabled = False
        Button_End.Enabled = True
        Button_Reset.Enabled = False
        Button_FastStop.Enabled = False
        Button_SlowStop.Enabled = False
        Button_DataSet.Enabled = False
        Button_Ccw_Scan.Enabled = False
        Button_Cw_Scan.Enabled = False
        Button_Org.Enabled = False
        Button_Rtn.Enabled = False
        Button_Ccw_Index.Enabled = False
        Button_Cw_Index.Enabled = False
        Button_Demo.Enabled = False
        Button_ClrError.Enabled = False
        Label_NowAddress.Text = "Please open the device."
    End Sub

    '********************************************************************************************************************
    '
    '   The processing that the Clear Error Button was clicked.
    '
    '********************************************************************************************************************
    Private Sub Command_ClrError_Click(sender As Object, e As EventArgs) Handles Button_ClrError.Click
        Dim sResult As New MC07_S_RESULT()

        sResult.Initialize()

        If (MC07_ClrError(hDev, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_ClrError() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        Label_Message.Text = "MC07_ClrError() succeed"

    End Sub

    '****************************************************************************************************************
    '
    '   The processing that the -Index Button was clicked.
    '
    '****************************************************************************************************************
    Private Sub Command_Ccw_Index_Click(sender As Object, e As EventArgs) Handles Button_Ccw_Index.Click
        Dim sResult As New MC07_S_RESULT()
        Dim Cmd As Short
        Dim LData As Integer
        Dim StopCode As Byte

        sResult.Initialize()
        Label_Message.Text = ""

        '*** INCREMENTAL INDEX DRIVE COMMAND ***
        If (CheckStatus1(hDev) = False) Then Exit Sub
        Cmd = &H14S
        LData = -3000
        If (MC07_LWDrive(hDev, Cmd, LData, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_LWDrive(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        Call ButtonDisable()
        Call ReadyWait(hDev, StopCode, False)
        Call StopCodeDisplay(StopCode)
        Call ButtonEnable()
    End Sub

    '****************************************************************************************************************
    '
    '   The processing that the +Index Button was clicked.
    '
    '****************************************************************************************************************
    Private Sub Command_Cw_Index_Click(sender As Object, e As EventArgs) Handles Button_Cw_Index.Click
        Dim sResult As New MC07_S_RESULT()
        Dim Cmd As Short
        Dim LData As Integer
        Dim StopCode As Byte

        sResult.Initialize()
        Label_Message.Text = ""

        '*** INCREMENTAL INDEX DRIVE COMMAND ***
        If (CheckStatus1(hDev) = False) Then Exit Sub
        Cmd = &H14S
        LData = 3000
        If (MC07_LWDrive(hDev, Cmd, LData, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_LWDrive(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        Call ButtonDisable()
        Call ReadyWait(hDev, StopCode, False)
        Call StopCodeDisplay(StopCode)
        Call ButtonEnable()
    End Sub

    '****************************************************************************************************************
    '
    '   The processing that the -Scan Button was clicked.
    '
    '****************************************************************************************************************
    Private Sub Command_Ccw_Scan_Click(sender As Object, e As EventArgs) Handles Button_Ccw_Scan.Click
        Dim sResult As New MC07_S_RESULT()
        Dim Cmd As Short
        Dim StopCode As Byte

        sResult.Initialize()
        Label_Message.Text = ""

        '*** -CCW SCAN DRIVE COMMAND ***
        If (CheckStatus1(hDev) = False) Then Exit Sub
        Cmd = &H13S
        If (MC07_BWDriveCommand(hDev, Cmd, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BWDriveCommand(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        Call ButtonDisable()
        Call ReadyWait(hDev, StopCode, False)
        Call StopCodeDisplay(StopCode)
        Call ButtonEnable()
    End Sub

    '****************************************************************************************************************
    '
    '   The processing that the +Scan Button was clicked.
    '
    '****************************************************************************************************************
    Private Sub Command_Cw_Scan_Click(sender As Object, e As EventArgs) Handles Button_Cw_Scan.Click
        Dim sResult As New MC07_S_RESULT()
        Dim Cmd As Short
        Dim StopCode As Byte

        sResult.Initialize()
        Label_Message.Text = ""

        '*** +CW SCAN DRIVE COMMAND ***
        If (CheckStatus1(hDev) = False) Then Exit Sub
        Cmd = &H12S
        If (MC07_BWDriveCommand(hDev, Cmd, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BWDriveCommand(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        Call ButtonDisable()
        Call ReadyWait(hDev, StopCode, False)
        Call StopCodeDisplay(StopCode)
        Call ButtonEnable()
    End Sub

    '****************************************************************************************************************
    '
    '   The processing that the Org Button was clicked.
    '
    '****************************************************************************************************************
    Private Sub Command_Org_Click(sender As Object, e As EventArgs) Handles Button_Org.Click
        Dim sResult As New MC07_S_RESULT()
        Dim StopCode As Byte

        sResult.Initialize()
        Label_Message.Text = ""
        '*** ORIGIN DRIVE ***
        If (CheckStatus1(hDev) = False) Then Exit Sub
        If (MC07_Org(hDev, MC07_ORG3, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_Org() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If
        Call ButtonDisable()
        Call ReadyWait(hDev, StopCode, True)                    ' Mcc07 Ready Wait
        Call StopCodeDisplay(StopCode)                          ' Stoppage factor of DRIVE is displayed.
        Call ButtonEnable()
    End Sub

    '****************************************************************************************************************
    '
    '   The processing that the Reset Button was clicked.
    '
    '****************************************************************************************************************
    Private Sub Command_Reset_Click(sender As Object, e As EventArgs) Handles Button_Reset.Click
        Dim sResult As New MC07_S_RESULT()
        Dim LData As Integer
        Dim Cmd As Short

        sResult.Initialize()
        Label_Message.Text = ""

        '*** ADDRESS COUNTER PRESET ***
        Cmd = &H80S
        LData = 0
        If (MC07_LWDrive(hDev, Cmd, LData, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_LWDrive(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        Call NowAddressDisplay(hDev)
    End Sub

    '****************************************************************************************************************
    '
    '   The processing that the Rtn Button was clicked.
    '
    '****************************************************************************************************************
    Private Sub Command_Rtn_Click(sender As Object, e As EventArgs) Handles Button_Rtn.Click
        Dim sResult As New MC07_S_RESULT()
        Dim Cmd As Short
        Dim LData As Integer
        Dim StopCode As Byte

        sResult.Initialize()
        Label_Message.Text = ""

        '*** ABSOLUTE INDEX DRIVE COMMAND ***
        If (CheckStatus1(hDev) = False) Then Exit Sub
        Cmd = &H15S
        LData = 0
        If (MC07_LWDrive(hDev, Cmd, LData, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_LWDrive(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If
        Call ButtonDisable()
        Call ReadyWait(hDev, StopCode, False)
        Call StopCodeDisplay(StopCode)
        Call ButtonEnable()
    End Sub

    '****************************************************************************************************************
    '
    '   The processing that the Stop Button was clicked.
    '
    '****************************************************************************************************************
    Private Sub Command_FastStop_Click(sender As Object, e As EventArgs) Handles Button_FastStop.Click
        Dim sResult As New MC07_S_RESULT()
        Dim Cmd As Short

        sResult.Initialize()
        '*** FAST STOP COMMAND ***
        Cmd = &HFFS
        If (MC07_BWDriveCommand(hDev, Cmd, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BWDriveCommand(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

    End Sub

    '****************************************************************************************************************
    '
    '   The processing that the Stop Button was clicked.
    '
    '****************************************************************************************************************
    Private Sub Command_SlowStop_Click(sender As Object, e As EventArgs) Handles Button_SlowStop.Click
        Dim sResult As New MC07_S_RESULT()
        Dim Cmd As Short

        sResult.Initialize()
        '*** SLOW STOP COMMAND ***
        Cmd = &HFES
        If (MC07_BWDriveCommand(hDev, Cmd, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BWDriveCommand(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

    End Sub


    '****************************************************************************************************************
    '
    '   The processing that the Demo Button was clicked.
    '
    '****************************************************************************************************************
    Private Sub Command_Demo_Click(sender As Object, e As EventArgs) Handles Button_Demo.Click
        Dim sResult As New MC07_S_RESULT()
        Dim n As Short
        Dim LData As Integer
        Dim Cmd As Short
        Dim StopCode As Byte

        sResult.Initialize()
        Label_Message.Text = ""
        Call ButtonDisable()

        If (CheckStatus1(hDev) = False) Then
            Call ButtonEnable()
            Exit Sub
        End If
        '*** ORIGIN DRIVE ***
        If (MC07_Org(hDev, MC07_ORG3, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_Org() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Call ButtonEnable()
            Exit Sub
        End If
        Call ReadyWait(hDev, StopCode, True)
        If StopCode <> 0 Then
            Call StopCodeDisplay(StopCode)
            Call ButtonEnable()
            Exit Sub
        End If

        '*** ADDRESS COUNTER PRESET ***
        Cmd = &H80S
        LData = 0
        If (MC07_LWDrive(hDev, Cmd, LData, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_LWDrive(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Call ButtonEnable()
            Exit Sub
        End If

        '*** INCREMENTAL INDEX DRIVE 4 Times ***
        For n = 0 To 3
            If (CheckStatus1(hDev) = False) Then
                Call ButtonEnable()
                Exit Sub
            End If

            Cmd = &H14S
            LData = 4000
            If (MC07_LWDrive(hDev, Cmd, LData, sResult) = False) Then
                Label_Message.Text = "It failed in MC07_LWDrive(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
                Call ButtonEnable()
                Exit Sub
            End If

            Call ReadyWait(hDev, StopCode, False)
            If StopCode <> 0 Then
                Call StopCodeDisplay(StopCode)
                Call ButtonEnable()
                Exit Sub
            End If
        Next

        '*** ABSOLUTE INDEX DRIVE ***
        If (CheckStatus1(hDev) = False) Then
            Call ButtonEnable()
            Exit Sub
        End If

        Cmd = &H15S
        LData = 30000
        If (MC07_LWDrive(hDev, Cmd, LData, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_LWDrive(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Call ButtonEnable()
            Exit Sub
        End If

        Call ReadyWait(hDev, StopCode, False)
        If StopCode <> 0 Then
            Call StopCodeDisplay(StopCode)
            Call ButtonEnable()
            Exit Sub
        End If

        '*** ABSOLUTE INDEX DRIVE ***
        If (CheckStatus1(hDev) = False) Then
            Call ButtonEnable()
            Exit Sub
        End If

        Cmd = &H15S
        LData = 0
        If (MC07_LWDrive(hDev, Cmd, LData, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_LWDrive(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Call ButtonEnable()
            Exit Sub
        End If

        Call ReadyWait(hDev, StopCode, False)
        Call StopCodeDisplay(StopCode)
        Call ButtonEnable()

    End Sub

    '****************************************************************************************************************
    '
    '   The processing that the Data Set Button was clicked.
    '
    '****************************************************************************************************************
    Private Sub Command_DataSet_Click(sender As Object, e As EventArgs) Handles Button_DataSet.Click
        Dim sResult As New MC07_S_RESULT()
        Dim Data As Short
        Dim LData As Integer
        Dim Cmd As Short

        sResult.Initialize()
        Label_Message.Text = ""

        '*** FSPD SET COMMAND       : 5000Hz ***
        If (CheckStatus1(hDev) = False) Then Exit Sub
        Cmd = &H5S
        LData = 5000
        If (MC07_LWDrive(hDev, Cmd, LData, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_LWDrive(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        '*** JSPD SET COMMAND       : 1000Hz ***
        If (CheckStatus1(hDev) = False) Then Exit Sub
        Cmd = &HCS
        LData = 1000
        If (MC07_LWDrive(hDev, Cmd, LData, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_LWDrive(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        '*** HIGH SPEED SET COMMAND ***
        If (CheckStatus1(hDev) = False) Then Exit Sub
        Data = 5000S                ' HSPD       : 5000Hz
        If (MC07_BWDriveData1(hDev, Data, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BWDriveData1() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        Data = &H3S                 ' RESOL No.  : 3
        If (MC07_BWDriveData2(hDev, Data, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BWDriveData2() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        Cmd = &H6S
        If (MC07_BWDriveCommand(hDev, Cmd, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BWDriveCommand(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        '*** LOW SPEED SET COMMAND ***
        If (CheckStatus1(hDev) = False) Then Exit Sub
        Data = 1000S                ' LSPD       : 1000Hz
        If (MC07_BWDriveData1(hDev, Data, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BWDriveData1() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        Data = 1000S                 ' ELSPD      : 1000Hz
        If (MC07_BWDriveData2(hDev, Data, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BWDriveData2() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        Cmd = &H7S
        If (MC07_BWDriveCommand(hDev, Cmd, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BWDriveCommand(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        '*** RATE SET COMMAND ***
        If (CheckStatus1(hDev) = False) Then Exit Sub
        Data = 200S                ' UCYCLE     : 100μs
        If (MC07_BWDriveData1(hDev, Data, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BWDriveData1() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        Data = 200S                 ' DCYCLE     : 100μs
        If (MC07_BWDriveData2(hDev, Data, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BWDriveData2() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        Cmd = &H8S
        If (MC07_BWDriveCommand(hDev, Cmd, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_BWDriveCommand(H'" & Hex(Cmd) & ") : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        '*** ORIGIN SPEC SET ***
        If (CheckStatus1(hDev) = False) Then Exit Sub
        If (MC07_SetOrgSpec(hDev, &H8000S, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_SetOrgSpec() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        '*** MARGIN PULSE            : 5 Pulse ***
        If (CheckStatus1(hDev) = False) Then Exit Sub
        If (MC07_SetOrgMarginPulse(hDev, 5, sResult) = False) Then
            Label_Message.Text = "It failed in MC07_SetOrgMarginPulse() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        Label_Message.Text = "DATA SET Succeed"


    End Sub

    '****************************************************************************************************************
    '
    '   The processing that the End Button was clicked.
    '
    '****************************************************************************************************************
    Private Sub Command_End_Click(sender As Object, e As EventArgs) Handles Button_End.Click
        End
    End Sub

    '****************************************************************************************************************
    '
    '   The processing that the Form was constructed.
    '
    '****************************************************************************************************************
    Private Sub Smp_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        Button_Env.Enabled = True
        Button_Open.Enabled = False
        Button_Close.Enabled = False
        Button_End.Enabled = True
        Button_Reset.Enabled = False
        Button_FastStop.Enabled = False
        Button_SlowStop.Enabled = False
        Button_DataSet.Enabled = False
        Button_Ccw_Scan.Enabled = False
        Button_Cw_Scan.Enabled = False
        Button_Org.Enabled = False
        Button_Rtn.Enabled = False
        Button_Ccw_Index.Enabled = False
        Button_Cw_Index.Enabled = False
        Label_NowAddress.Text = "Please environmental setting."
    End Sub

    '****************************************************************************************************************
    '
    '   The processing that the Form was unload.
    '
    '****************************************************************************************************************
    Private Sub Smp_Closed(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles MyBase.Closed
        Dim sResult As New MC07_S_RESULT()
        Dim Ret As Boolean

        sResult.Initialize()

        Ret = MC07_BClose(hDev, sResult)

        Ret = MC07_UWExUnitCommControl(hUnit, MC07_EX_UNIT_COMM_STOP, sResult)

        Ret = MC07_UClose(hUnit, sResult)

        Ret = MC07_BPortClose(hInPort, sResult)

        Ret = MC07_BPortClose(hOutPort, sResult)

    End Sub
    '********************************************************************************************************************
    '
    '   Check ExtUnitStatus.
    '   Parameter UnitHandle         ･･･ Unit Handle
    '   Return Value              ･･･ TRUE: CONNECT BIT = 1 and DISCONNECT LATCH BIT = 0
    '                                 FALSE: CONNECT BIT = 0 or DISCONNECT LATCH BIT = 1
    '********************************************************************************************************************
    Private Function CheckExtUnitStatus(ByVal UnitHandle As Long) As Boolean
        Dim sResult As New MC07_S_RESULT()
        Dim extUnitStatus As Integer

        sResult.Initialize()
        '*** READ UNIT COMMUNICATION STATUS ***
        If (MC07_URExUnitCommStatus(UnitHandle, extUnitStatus, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_URExUnitCommStatus() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Return False
        End If

        '*** CHECK CONNECT BIT ***
        If ((extUnitStatus And &H2S) = 0) Then
            Label_Message_ExtIO.Text = "EXTEND UNIT STATUS is not Connected." & vbCrLf & " Please Check EXTEND UNIT connection"
            Return False
        End If

        '*** CHECK DISCONNECT LATCH BIT ***
        If (extUnitStatus And &H4S) Then
            Label_Message_ExtIO.Text = "EXTEND UNIT STATUS is DISCONNECT LATCH." & vbCrLf & "Please Click 'Latch Clear' button"
            Return False
        End If

        Return True

    End Function

    '********************************************************************************************************************
    '
    '   The processing that the Open Extend IO was clicked.
    '
    '********************************************************************************************************************
    Private Sub Command_OpenExtIO_Click(sender As Object, e As EventArgs) Handles Button_OpenExtIO.Click
        Dim sResult As New MC07_S_RESULT()
        Dim extUnitStatus As Short
        Dim Ret As Boolean
        Dim Data As Short

        sResult.Initialize()

        Label_Message_ExtIO.Text = ""
        '*** UNIT OPEN ***
        If (MC07_UOpen(MC07_USB_UNIT_0, hUnit, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_UOpen() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Button_Env.Enabled = True
            Exit Sub
        End If

        '*** UNIT COMMUNICATION MODE ***
        If (MC07_UWExUnitCommMode(hUnit, MC07_EX_UNIT_COMM_RATE_5, 3, MC07_EX_UNIT_COMM_16BIT, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_UWExUnitCommMode() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            'Because it is avoided double Unit Open Error
            Ret = MC07_UClose(hUnit, sResult)
            Button_Env.Enabled = True
            Exit Sub
        End If

        '*** UNIT COMMUNICATION CONTROL ***
        If (MC07_UWExUnitCommControl(hUnit, MC07_EX_UNIT_COMM_START, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_UWExUnitCommControl(START)" & vbCrLf & " : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            'Because it is avoided double Unit Open Error
            Ret = MC07_UClose(hUnit, sResult)
            Button_Env.Enabled = True
            Exit Sub
        End If

        '*** WAIT CONNECT BIT = 1 ***
        Do
            '*** READ UNIT COMMUNICATION STATUS ***
            If (MC07_URExUnitCommStatus(hUnit, extUnitStatus, sResult) = False) Then
                Label_Message_ExtIO.Text = "It failed in MC07_URExUnitCommStatus() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
                'Because it is avoided double Unit Open Error
                Ret = MC07_UClose(hUnit, sResult)
                Button_Env.Enabled = True
                Exit Sub
            End If

            '*** CHECK POLLING BIT ***
            If ((extUnitStatus And &H1S) = 0) Then
                Label_Message_ExtIO.Text = "EXTEND UNIT STATUS is not Polling." & vbCrLf & "Please Check EXTEND UNIT connection"
                'Because it is avoided double Unit Open Error
                Ret = MC07_UClose(hUnit, sResult)
                Exit Sub
            End If

            '*** CHECK DISCONNECT LATCH BIT ***
            If (extUnitStatus And &H4S) Then
                Label_Message_ExtIO.Text = "EXTEND UNIT STATUS is DISCONNECT LATCH." & vbCrLf & " Please Check EXTEND UNIT connection"

                '*** UNIT COMMUNICATION CONTROL ***
                If (MC07_UWExUnitCommControl(hUnit, MC07_EX_UNIT_COMM_DISC_LATCH_CLR, sResult) = False) Then
                    Label_Message_ExtIO.Text = "It failed in MC07_UWExUnitCommControl(LATCH_CLR) " & vbCrLf & ": MC07_Result(1)=" & Str(sResult.MC07_Result(1))
                    'Because it is avoided double Unit Open Error
                    Ret = MC07_UClose(hUnit, sResult)
                    Button_Env.Enabled = True
                    Exit Sub
                End If

                Ret = MC07_UClose(hUnit, sResult)
                Exit Sub
            End If

            '*** CHECK CONNECT BIT ***
        Loop While (extUnitStatus And &H2S) = 0

        '*** BPORT OPEN ***
        If (MC07_BPortOpen(MC07_USB_UNIT_0, MC07_EXP0_IN, hInPort, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_BPortOpen() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            'Because it is avoided double Unit Open Error
            Ret = MC07_UClose(hUnit, sResult)
            Button_Env.Enabled = True
            Exit Sub
        End If

        '*** BPORT OPEN ***
        If (MC07_BPortOpen(MC07_USB_UNIT_0, MC07_EXP0_OUT, hOutPort, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_BPortOpen() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            'Because it is avoided double Unit Open Error and double BPort Open Error 
            Ret = MC07_UClose(hUnit, sResult)
            Ret = MC07_BPortClose(hInPort, sResult)
            Button_Env.Enabled = True
            Exit Sub
        End If

        '*** READ BPORT OUT ***
        If (MC07_BPortIn(hOutPort, Data, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_BPortIn() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            'Because it is avoided double Unit Open Error and double BPort Open Error 
            Ret = MC07_UClose(hUnit, sResult)
            Ret = MC07_BPortClose(hInPort, sResult)
            Ret = MC07_BPortClose(hOutPort, sResult)
            Button_Env.Enabled = True
            Exit Sub
        End If

        If (Data And &H1S) Then Label_ExtOut0.Text = "ON" Else Label_ExtOut0.Text = "OFF"

        If (Data And &H2S) Then Label_ExtOut1.Text = "ON" Else Label_ExtOut1.Text = "OFF"

        If (Data And &H4S) Then Label_ExtOut2.Text = "ON" Else Label_ExtOut2.Text = "OFF"

        If (Data And &H8S) Then Label_ExtOut3.Text = "ON" Else Label_ExtOut3.Text = "OFF"

        '*** READ BPORT IN ***'
        Button_ReadExtIn0.Enabled = True
        Button_ReadExtIn0.PerformClick()

        Button_OpenExtIO.Enabled = False
        Button_CloseExtIO.Enabled = True
        Button_ExtLatchClr.Enabled = True
        Button_ExtOut0.Enabled = True
        Button_ExtOut1.Enabled = True
        Button_ExtOut2.Enabled = True
        Button_ExtOut3.Enabled = True

    End Sub

    '********************************************************************************************************************
    '
    '   The processing that the Close Extend IO was clicked.
    '
    '********************************************************************************************************************
    Private Sub Command_CloseExtIO_Click(sender As Object, e As EventArgs) Handles Button_CloseExtIO.Click

        Dim sResult As New MC07_S_RESULT()

        sResult.Initialize()
        '*** UNIT COMMUNICATION CONTROL ***
        If (MC07_UWExUnitCommControl(hUnit, MC07_EX_UNIT_COMM_STOP, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_UWExUnitCommControl(STOP) " & vbCrLf & ": MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Button_Env.Enabled = True
            Exit Sub
        End If

        '*** UNIT CLOSE ***
        If (MC07_UClose(hUnit, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_UClose() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        '*** BPORT CLOSE ***
        If (MC07_BPortClose(hInPort, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_BPortClose() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        '*** BPORT CLOSE ***
        If (MC07_BPortClose(hOutPort, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_BPortClose() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        Label_Message_ExtIO.Text = ""
        Label_ExtOut0.Text = "" : Label_ExtOut1.Text = "" : Label_ExtOut2.Text = "" : Label_ExtOut3.Text = ""
        Label_ExtIn0.Text = "" : Label_ExtIn1.Text = "" : Label_ExtIn2.Text = "" : Label_ExtIn3.Text = ""

        Button_Env.Enabled = True
        Button_OpenExtIO.Enabled = True
        Button_CloseExtIO.Enabled = False
        Button_ExtLatchClr.Enabled = False
        Button_ReadExtIn0.Enabled = False
        Button_ExtOut0.Enabled = False
        Button_ExtOut1.Enabled = False
        Button_ExtOut2.Enabled = False
        Button_ExtOut3.Enabled = False

    End Sub

    '********************************************************************************************************************
    '
    '   The processing that the Latch Clear was clicked.
    '
    '********************************************************************************************************************
    Private Sub Command_ExtLatchClr_Click(sender As Object, e As EventArgs) Handles Button_ExtLatchClr.Click

        Dim sResult As New MC07_S_RESULT()

        sResult.Initialize()

        '*** UNIT COMMUNICATION CONTROL ***
        If (MC07_UWExUnitCommControl(hUnit, MC07_EX_UNIT_COMM_DISC_LATCH_CLR, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_UWExUnitCommControl(LATCH_CLR)" & vbCrLf & ": MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        Label_Message_ExtIO.Text = "DISCONNECT LATCH CLEAR succeed"

    End Sub


    '********************************************************************************************************************
    '
    '   The processing that the Extend Unit I/O OUT00 was clicked.
    '
    '********************************************************************************************************************
    Private Sub Command_ExtOut0_Click(sender As Object, e As EventArgs) Handles Button_ExtOut0.Click
        Dim sResult As New MC07_S_RESULT()
        Dim Data As Short

        sResult.Initialize()
        Label_Message_ExtIO.Text = ""

        '*** READ BPORT OUT ***
        If (MC07_BPortIn(hOutPort, Data, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_BPortIn() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        If (Data And &H1S) Then
            'ON -> OFF
            Data = &HFFFFS Xor &H1S
            '*** BPORT AND OUT ***
            If (MC07_BPortAndOut(hOutPort, Data, sResult) = False) Then
                Label_Message_ExtIO.Text = "It failed in MC07_BPortAndOut() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
                Exit Sub
            End If

            'OFF -> ON
        Else
            Data = &H1S
            '*** BPORT OR OUT ***
            If (MC07_BPortOrOut(hOutPort, Data, sResult) = False) Then
                Label_Message_ExtIO.Text = "It failed in MC07_BPortOrOut() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
                Exit Sub
            End If
        End If

        '*** CHECK UNIT I/O CONNECTINON *****
        If (CheckExtUnitStatus(hUnit) = False) Then Exit Sub

        '*** READ BPORT OUT ***
        If (MC07_BPortIn(hOutPort, Data, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_BPortIn() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        If (Data And &H1S) Then Label_ExtOut0.Text = "ON" Else Label_ExtOut0.Text = "OFF"

    End Sub

    '********************************************************************************************************************
    '
    '   The processing that the Extend Unit I/O OUT01 was clicked.
    '
    '********************************************************************************************************************
    Private Sub Command_ExtOut1_Click(sender As Object, e As EventArgs) Handles Button_ExtOut1.Click
        Dim sResult As New MC07_S_RESULT()
        Dim Data As Short

        sResult.Initialize()
        Label_Message_ExtIO.Text = ""

        '*** READ BPORT OUT ***
        If (MC07_BPortIn(hOutPort, Data, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_BPortIn() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        If (Data And &H2S) Then
            'ON -> OFF
            Data = &HFFFFS Xor &H2S
            '*** BPORT AND OUT ***
            If (MC07_BPortAndOut(hOutPort, Data, sResult) = False) Then
                Label_Message_ExtIO.Text = "It failed in MC07_BPortAndOut() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
                Exit Sub
            End If

            'OFF -> ON
        Else
            Data = &H2S
            '*** BPORT OR OUT ***
            If (MC07_BPortOrOut(hOutPort, Data, sResult) = False) Then
                Label_Message_ExtIO.Text = "It failed in MC07_BPortOrOut() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
                Exit Sub
            End If
        End If

        '*** CHECK EXT UNIT I/O CONNECT *****
        If (CheckExtUnitStatus(hUnit) = False) Then Exit Sub

        '*** READ BPORT OUT ***
        If (MC07_BPortIn(hOutPort, Data, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_BPortIn() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        If (Data And &H2S) Then Label_ExtOut1.Text = "ON" Else Label_ExtOut1.Text = "OFF"

    End Sub

    '********************************************************************************************************************
    '
    '   The processing that the Extend Unit I/O OUT02 was clicked.
    '
    '********************************************************************************************************************
    Private Sub Command_ExtOut2_Click(sender As Object, e As EventArgs) Handles Button_ExtOut2.Click
        Dim sResult As New MC07_S_RESULT()
        Dim Data As Short

        sResult.Initialize()
        Label_Message_ExtIO.Text = ""

        '*** READ BPORT OUT ***
        If (MC07_BPortIn(hOutPort, Data, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_BPortIn() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        If (Data And &H4S) Then
            'ON -> OFF
            Data = &HFFFFS Xor &H4S
            '*** BPORT AND OUT ***
            If (MC07_BPortAndOut(hOutPort, Data, sResult) = False) Then
                Label_Message_ExtIO.Text = "It failed in MC07_BPortAndOut() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
                Exit Sub
            End If

            'OFF -> ON
        Else
            Data = &H4S
            '*** BPORT OR OUT ***
            If (MC07_BPortOrOut(hOutPort, Data, sResult) = False) Then
                Label_Message_ExtIO.Text = "It failed in MC07_BPortOrOut() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
                Exit Sub
            End If
        End If

        '*** CHECK EXT UNIT I/O CONNECT *****
        If (CheckExtUnitStatus(hUnit) = False) Then Exit Sub

        '*** READ BPORT OUT ***
        If (MC07_BPortIn(hOutPort, Data, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_BPortIn() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        If (Data And &H4S) Then Label_ExtOut2.Text = "ON" Else Label_ExtOut2.Text = "OFF"

    End Sub

    '********************************************************************************************************************
    '
    '   The processing that the Extend Unit I/O OUT03 was clicked.
    '
    '********************************************************************************************************************
    Private Sub Command_ExtOut3_Click(sender As Object, e As EventArgs) Handles Button_ExtOut3.Click
        Dim sResult As New MC07_S_RESULT()
        Dim Data As Short

        sResult.Initialize()
        Label_Message_ExtIO.Text = ""

        '*** READ BPORT IN ***
        If (MC07_BPortIn(hOutPort, Data, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_BPortIn() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        If (Data And &H8S) Then
            'ON -> OFF
            Data = &HFFFFS Xor &H8S
            '*** BPORT AND OUT ***
            If (MC07_BPortAndOut(hOutPort, Data, sResult) = False) Then
                Label_Message_ExtIO.Text = "It failed in MC07_BPortAndOut() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
                Exit Sub
            End If

            'OFF -> ON
        Else
            Data = &H8S
            '*** BPORT OR OUT ***
            If (MC07_BPortOrOut(hOutPort, Data, sResult) = False) Then
                Label_Message_ExtIO.Text = "It failed in MC07_BPortOrOut() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
                Exit Sub
            End If
        End If

        '*** CHECK EXT UNIT I/O CONNECT *****
        If (CheckExtUnitStatus(hUnit) = False) Then Exit Sub

        '*** READ BPORT OUT ***
        If (MC07_BPortIn(hOutPort, Data, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_BPortIn() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        If (Data And &H8S) Then Label_ExtOut3.Text = "ON" Else Label_ExtOut3.Text = "OFF"

    End Sub

    '********************************************************************************************************************
    '
    '   The processing that the Read Extend Unit was clicked.
    '
    '********************************************************************************************************************
    Private Sub Command_ReadExtIn0_Click(sender As Object, e As EventArgs) Handles Button_ReadExtIn0.Click
        Dim sResult As New MC07_S_RESULT()
        Dim Data As Short

        sResult.Initialize()
        Label_Message_ExtIO.Text = ""
        '*** CHECK EXT UNIT I/O CONNECT *****
        If (CheckExtUnitStatus(hUnit) = False) Then Exit Sub

        '*** READ BPORT IN ***
        If (MC07_BPortIn(hInPort, Data, sResult) = False) Then
            Label_Message_ExtIO.Text = "It failed in MC07_BPortIn() : MC07_Result(1)=" & Str(sResult.MC07_Result(1))
            Exit Sub
        End If

        If (Data And &H1S) Then Label_ExtIn0.Text = "ON" Else Label_ExtIn0.Text = "OFF"

        If (Data And &H2S) Then Label_ExtIn1.Text = "ON" Else Label_ExtIn1.Text = "OFF"

        If (Data And &H4S) Then Label_ExtIn2.Text = "ON" Else Label_ExtIn2.Text = "OFF"

        If (Data And &H8S) Then Label_ExtIn3.Text = "ON" Else Label_ExtIn3.Text = "OFF"


    End Sub

End Class
